# -*- coding: mbcs -*-
from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *

############################# START OF GEOMETRY-SCRIPT #################################
import numpy as np
import math
import subprocess
# Model geometry, input from textfile: "total_geometry.txt"
# The wire geometry is drawn from left end to right end, and the support is drawn from wall and out.
# The skip_footer/skip_header in the coord-lists, are dependant on how many coordinates each component includes.
# The number of coordinates for each component is fixed in this script, and should NOT be altered.
# Values for each coordinate may however be altered
# The dropper coordinates are taken as the two points between the support and the left/right end of, the system. 
# To further explain the dropper coordinates, they are taken as point 3-4, 5-6, 9-10, 11-12 in the
# excel file 'Total geometri'

##### Which droppers to suppress ####
drSuppressable = ['Wire-3','Wire-4','Wire-5','Wire-6']
# drSuppressable is the wire features for Dropper 1-4.
# To suppress a dropper, include the dropper position in dropperSuppress.
# For instance, to suppress Dropper 2 and Dropper 3, simply type dropperSuppress = [2,3]
# NB! Do not type dropperSuppress = [3,2], as the script don't understand this sequence. Lowest int first.
# Maximum droppers to suppress is dropperSuppress = [1,2,3,4]
# Default: No suppressed droppers is dropperSuppress = []
dropperSuppress = []

# Number of eigenvectors (modes) in frequency step
numEigenVec = 4 

# Number of coordinates for each component (droppers are excluded since they appear from MW/CW geometry):
numMW = 7               # Messenger wire
numCW = numMW           # Contact wire
numTopAnchor = 4        # Top Anchor
numCantTube = 3         # Cantilvered tube
numRegArmDropper = 2    # Registration arm dropper
numRegArm = 5           # Registration arm
numStdArm = 2           # Steady arm
numTot = numMW + numCW + numTopAnchor + \
         numCantTube + numRegArmDropper + \
         numRegArm + numStdArm
lengths = [numMW,numCW,numTopAnchor,numCantTube,\
          numRegArmDropper,numRegArm,numStdArm]

## Input coordinates for MW, CW, and support. See README-file for explanation of terms
mwcoord = np.genfromtxt('total_geometry.txt', comments='#', skip_footer=(numTot-numMW), delimiter=',')
mwcoord = mwcoord.tolist()
cwcoord = np.genfromtxt('total_geometry.txt', comments='#', skip_header=(14), \
                        skip_footer=(numTot-numMW-numCW), delimiter=',')
cwcoord = cwcoord.tolist()
tacoord = np.genfromtxt('total_geometry.txt', comments='#', skip_header=(26), \
                        skip_footer=(numTot-numMW-numCW-numTopAnchor), delimiter=',')
tacoord = tacoord.tolist()
ctcoord = np.genfromtxt('total_geometry.txt', comments='#', skip_header=(32), \
                        skip_footer=(numTot-numMW-numCW-numTopAnchor-numCantTube), delimiter=',')
ctcoord = ctcoord.tolist()
radcoord = np.genfromtxt('total_geometry.txt', comments='#', skip_header=(37), \
                        skip_footer=(numTot-numMW-numCW-numTopAnchor-numCantTube-numRegArmDropper), delimiter=',')
radcoord = radcoord.tolist()
racoord = np.genfromtxt('total_geometry.txt', comments='#', skip_header=(41), \
                        skip_footer=(numStdArm), delimiter=',')
racoord = racoord.tolist()
sacoord = np.genfromtxt('total_geometry.txt', comments='#', skip_header=(48), \
                        delimiter=',')
sacoord = sacoord.tolist()

# All wires/support components (except droppers) stored in a 'vector' list_of_components
list_of_components = [mwcoord,cwcoord,tacoord,ctcoord,radcoord,racoord,sacoord]     

######### Creating coordinates for sets #######
midCoordNames = ['mwMidpoint','cwMidpoint','taMidpoint' \
'ctMidpoint','radMidpoint','raMidpoint','saMidpoint']
midCoordinates = list()
for i in range(0,len(lengths)):
    midCoordinates.append([[1]*3]*(lengths[i]-1))
    for j in range(0,len(midCoordinates[i])):
        midCoordinates[i][j] = 0.5*np.add(list_of_components[i][j],list_of_components[i][j+1])
            
for i in range(0,len(lengths)):
    for j in range(0,len(midCoordinates[i])):
            midCoordinates[i][j] = tuple(midCoordinates[i][j])
for i in range(0,len(lengths)):
    midCoordinates[i] = tuple(midCoordinates[i])
mwMidpoint,cwMidpoint,taMidpoint, \
ctMidpoint,radMidpoint,raMidpoint,saMidpoint = tuple(midCoordinates)
######## END creating coordinates for sets #######

for i in range(len(list_of_components)):                                         # Edited all component lists to fit as 'polyline wire' in Abaqus
    for j in range(1,len(list_of_components[i])*2-3,2):
        list_of_components[i].insert(j,list_of_components[i][j])                    
        
for i in range(len(list_of_components)):                                         # Making the lists a tuple, to operate correctly in Abaqus
    k=0  
    while (k < len(list_of_components[i])):
        list_of_components[i][k] = tuple(list_of_components[i][k])		                                    
        k = k+1
        
# Making the tuple in the arrangement Abaqus wants: ((coordinate1,coordinate2),(coordinate2,coordinate3)) etc for all components:
real_list_of_components = [0]*len(list_of_components)
for i in range(len(real_list_of_components)):
    real_list_of_components[i] = [0]*(lengths[i]-1)
[mwCoordinates,cwCoordinates,taCoordinates,ctCoordinates,radCoordinates,raCoordinates,saCoordinates] = real_list_of_components
for i in range(len(real_list_of_components)):
    k=0
    while (k < (int(len(real_list_of_components[i])))):
           real_list_of_components[i][k] = tuple(list_of_components[i][k*2:k*2+2])
           k=k+1

## Droppers geometry - drCoordinates - gained from mw/cw
numDr = 4 #Number of droppers included
drCoord = [0]*10
for i in range(1,len(mwcoord)/2-1,2) + range(7,len(mwcoord)-1,2):
    drCoord[i-1] = cwcoord[i]
    drCoord[i] = mwcoord[i]
del drCoord[4:6]

drCoordinates = [0]*numDr
drNames = ['Dropper 1','Dropper 2','Dropper 3','Dropper 4']
k = 0
while (k <= (len(drCoord)/2-1)):
    drCoordinates[k] = tuple(drCoord[k*2:k*2+2])
    k=k+1

########################################################################
############ Creating curve for registration arm dropper ############### NEW
########################################################################
numRegAD = 10   # Number of points on registration arm dropper
# Edited in positive global x-direction (negative y-dir)
#used radcoord for registration arm dropper coordinates
radDefAmplitude = 0.01     #Deflection amplitude
radcoord = [[radcoord[0][0],radcoord[0][1],radcoord[0][2]],\
            [radcoord[1][0],radcoord[1][1],radcoord[1][2]]]            
radXLength = radcoord[1][0]-radcoord[0][0]
radYLength = radcoord[1][1]-radcoord[0][1]
radLength = math.sqrt(radXLength**2+radYLength**2)
angle = math.atan(radXLength/(radYLength*(-1)))
radDX = np.linspace(0,radXLength,numRegAD)
radDY = np.linspace(0,radYLength,numRegAD)
radDTot = [0]*len(radDX)
radDSin = [0]*len(radDX)
radDeltaX = [0]*len(radDX)
radDeltaY = [0]*len(radDY)
radCoordinates = list()
for i in range(0,len(radDX)):
    radDTot[i] =math.sqrt(radDX[i]**2+radDY[i]**2)
    radDSin[i] =math.sin(radDTot[i]*np.pi/radLength)
    radDeltaX[i] = math.cos(angle)*radDSin[i]*radDefAmplitude
    radDeltaY[i]= math.sin(angle)*radDSin[i]*radDefAmplitude
for i in range(0,(len(radDeltaX))):
    radCoordinates.append([radcoord[0][0]+radDeltaX[i]+radDX[i],radcoord[0][1]-radDeltaY[i]+radDY[i]])
for i in range(0,len(radCoordinates)):
    radCoordinates[i].append(0)


##########################################################################################
############################ Creating geometric error in droppers#########################
##########################################################################################
# Defining lengths of each dropper
drLength=list()
drDeflection = 0.005
drNumDefPoints = 50                        #Deflectionpoints
drCreateMidPoints = 2*drNumDefPoints - 1   #Midpoints on dropper
# Defining dropper coordinates to be able to create sin-wave geometry
drXCoordinates = list()
drYCoordinates = list()
drZCoordinates = list()
drY = list()
drXTotCoordinates = list()
drYTotCoordinates = list()
drZTotCoordinates = list()
drYTot = list()
for i in range(0,len(drCoordinates)):
    # Deflectionpoints
    drLength.append(drCoordinates[i][1][1]-drCoordinates[i][0][1])
    drY.append(np.linspace(0,drLength[i],drNumDefPoints))
    drXCoordinates.append(np.sin(drY[i]*np.pi/(drLength[i]))*drDeflection)
    drXCoordinates[i][-1] = 0
    drXCoordinates[i][0] = 0
    drXCoordinates[i] = drXCoordinates[i]+drCoordinates[i][0][0]
    drYCoordinates.append(drY[i] + drCoordinates[i][0][1])
    drZCoordinates.append([drCoordinates[i][0][2]]*len(drYCoordinates[0]))
    # Midpoints
    drYTot.append(np.linspace(0,drLength[i],drCreateMidPoints))
    drXTotCoordinates.append(np.sin(drYTot[i]*np.pi/(drLength[i]))*drDeflection)
    drXTotCoordinates[i][-1] = 0
    drXTotCoordinates[i][0] = 0
    drXTotCoordinates[i] = drXTotCoordinates[i]+drCoordinates[i][0][0]
    drYTotCoordinates.append(drYTot[i] + drCoordinates[i][0][1])
    drZTotCoordinates.append([drCoordinates[i][0][2]]*len(drYTotCoordinates[0]))

#Deflectionpoints
dr1SkewGeometry = [[0]*3]*(len(drXCoordinates[0])*2)
dr2SkewGeometry = [[0]*3]*(len(drXCoordinates[0])*2)
dr3SkewGeometry = [[0]*3]*(len(drXCoordinates[0])*2)
dr4SkewGeometry = [[0]*3]*(len(drXCoordinates[0])*2)

################# Making tuple for the skew geometry of the droppers ######################
# First dropper
for j in range(1,len(dr1SkewGeometry),2):
    dr1SkewGeometry[j-1] = [drXCoordinates[0][int(j/2)],drYCoordinates[0][int(j/2)],drZCoordinates[0][int(j/2)]]
    dr1SkewGeometry[j] = [drXCoordinates[0][int(j/2)],drYCoordinates[0][int(j/2)],drZCoordinates[0][int(j/2)]]
    dr1SkewGeometry[j-1] = tuple(dr1SkewGeometry[j-1])
    dr1SkewGeometry[j] = tuple(dr1SkewGeometry[j])
del dr1SkewGeometry[0]
del dr1SkewGeometry[-1]

for i in range(0,len(dr1SkewGeometry)):
        dr1SkewGeometry[i] = tuple(dr1SkewGeometry[i*2:i*2+2])
del dr1SkewGeometry[int(len(dr1SkewGeometry)/2):-1]
del dr1SkewGeometry[-1]
# Second dropper
for j in range(1,len(dr2SkewGeometry),2):
    dr2SkewGeometry[j-1] = [drXCoordinates[1][int(j/2)],drYCoordinates[1][int(j/2)],drZCoordinates[1][int(j/2)]]
    dr2SkewGeometry[j] = [drXCoordinates[1][int(j/2)],drYCoordinates[1][int(j/2)],drZCoordinates[1][int(j/2)]]
    dr2SkewGeometry[j-1] = tuple(dr2SkewGeometry[j-1])
    dr2SkewGeometry[j] = tuple(dr2SkewGeometry[j])
del dr2SkewGeometry[0]
del dr2SkewGeometry[-1]

for i in range(0,len(dr2SkewGeometry)):
        dr2SkewGeometry[i] = tuple(dr2SkewGeometry[i*2:i*2+2])
del dr2SkewGeometry[int(len(dr2SkewGeometry)/2):-1]
del dr2SkewGeometry[-1]

# Third dropper
for j in range(1,len(dr3SkewGeometry),2):
    dr3SkewGeometry[j-1] = [drXCoordinates[2][int(j/2)],drYCoordinates[2][int(j/2)],drZCoordinates[2][int(j/2)]]
    dr3SkewGeometry[j] = [drXCoordinates[2][int(j/2)],drYCoordinates[2][int(j/2)],drZCoordinates[2][int(j/2)]]
    dr3SkewGeometry[j-1] = tuple(dr3SkewGeometry[j-1])
    dr3SkewGeometry[j] = tuple(dr3SkewGeometry[j])
del dr3SkewGeometry[0]
del dr3SkewGeometry[-1]

for i in range(0,len(dr3SkewGeometry)):
        dr3SkewGeometry[i] = tuple(dr3SkewGeometry[i*2:i*2+2])
del dr3SkewGeometry[int(len(dr3SkewGeometry)/2):-1]
del dr3SkewGeometry[-1]

# Fourth dropper
for j in range(1,len(dr4SkewGeometry),2):
    dr4SkewGeometry[j-1] = [drXCoordinates[3][int(j/2)],drYCoordinates[3][int(j/2)],drZCoordinates[3][int(j/2)]]
    dr4SkewGeometry[j] = [drXCoordinates[3][int(j/2)],drYCoordinates[3][int(j/2)],drZCoordinates[3][int(j/2)]]
    dr4SkewGeometry[j-1] = tuple(dr4SkewGeometry[j-1])
    dr4SkewGeometry[j] = tuple(dr4SkewGeometry[j])
del dr4SkewGeometry[0]
del dr4SkewGeometry[-1]

for i in range(0,len(dr4SkewGeometry)):
        dr4SkewGeometry[i] = tuple(dr4SkewGeometry[i*2:i*2+2])
del dr4SkewGeometry[int(len(dr4SkewGeometry)/2):-1]
del dr4SkewGeometry[-1]


drSkewGeometry =[dr1SkewGeometry,dr2SkewGeometry,dr3SkewGeometry,dr4SkewGeometry]

# Making input for the midpoints of the droppers (to create dropper sets with)
dr1Midpoint = [[0]*3]*(len(drXTotCoordinates[0])/2)
dr2Midpoint = [[0]*3]*(len(drXTotCoordinates[1])/2)
dr3Midpoint = [[0]*3]*(len(drXTotCoordinates[2])/2)
dr4Midpoint = [[0]*3]*(len(drXTotCoordinates[3])/2)

for j in range(0,len(dr1Midpoint)):
    dr1Midpoint[j] = [drXTotCoordinates[0][j*2],drYTotCoordinates[0][j*2],drZTotCoordinates[0][j*2]]
dr1Midpoint.append([drXTotCoordinates[0][-2],drYTotCoordinates[0][-2],drZTotCoordinates[0][-2]])
for j in range(0,len(dr2Midpoint)):
    dr2Midpoint[j] = [drXTotCoordinates[1][j*2],drYTotCoordinates[1][j*2],drZTotCoordinates[1][j*2]]
dr2Midpoint.append([drXTotCoordinates[1][-2],drYTotCoordinates[1][-2],drZTotCoordinates[1][-2]])
for j in range(0,len(dr3Midpoint)):
    dr3Midpoint[j] = [drXTotCoordinates[2][j*2],drYTotCoordinates[2][j*2],drZTotCoordinates[2][j*2]]
dr3Midpoint.append([drXTotCoordinates[2][-2],drYTotCoordinates[2][-2],drZTotCoordinates[2][-2]])
for j in range(0,len(dr4Midpoint)):
    dr4Midpoint[j] = [drXTotCoordinates[3][j*2],drYTotCoordinates[3][j*2],drZTotCoordinates[3][j*2]]
dr4Midpoint.append([drXTotCoordinates[3][-2],drYTotCoordinates[3][-2],drZTotCoordinates[3][-2]])

####################### Geometry for cantilevers #######################################
z_offset = 0.05 	#5cm offset in longitudinal direction
z_left = cwCoordinates[0][0][2] + z_offset
z_right = cwCoordinates[-1][-1][2] - z_offset
left_cw_cant = [[taCoordinates[0][0][0],cwCoordinates[0][0][1],z_left],[cwCoordinates[0][0][0],\
               cwCoordinates[0][0][1],z_left],[2.4,cwCoordinates[0][0][1],z_left]]
left_mw_cant = [[taCoordinates[0][0][0],mwCoordinates[0][0][1],z_left],[mwCoordinates[0][0][0],\
               mwCoordinates[0][0][1],z_left],[2.4,mwCoordinates[0][0][1],z_left]]
right_mw_cant = [[taCoordinates[0][0][0],mwCoordinates[-1][-1][1],z_right],[mwCoordinates[-1][-1][0],\
               mwCoordinates[-1][-1][1],z_right],[2.4,mwCoordinates[-1][-1][1],z_right]]
right_cw_cant = [[taCoordinates[0][0][0],cwCoordinates[-1][-1][1],z_right],[cwCoordinates[-1][-1][0],\
               cwCoordinates[-1][-1][1],z_right],[2.4,cwCoordinates[-1][-1][1],z_right]]

cantCoordinates =[left_cw_cant,left_mw_cant,right_cw_cant,right_mw_cant]
cantNames = ['Left CW Cantilever','Left MW cantilever','Right CW Cantilever','Right MW Cantilever']
for i in range(0,len(cantCoordinates)):                                         # Edited all component lists to fit as 'polyline wire' in Abaqus
    for j in range(1,len(cantCoordinates[i])*2-3,2):
        cantCoordinates[i].insert(j,cantCoordinates[i][j])
for i in range(len(list_of_components)):                                         # Making the lists a tuple, to operate correctly in Abaqus
    k=0  
    while (k < len(list_of_components[i])):
        list_of_components[i][k] = tuple(list_of_components[i][k])		                                    
        k = k+1
        
# Making the tuple in the arrangement Abaqus wants: ((coordinate1,coordinate2),(coordinate2,coordinate3)) etc for all components:
real_cantCoordinates= [0]*len(cantCoordinates)
for i in range(len(real_cantCoordinates)):
    real_cantCoordinates[i] = [0]*2

for i in range(len(real_cantCoordinates)):
    k=0
    while (k < 2):
           real_cantCoordinates[i][k] = tuple(cantCoordinates[i][k*2:k*2+2])
           k=k+1
cantCoordinates = real_cantCoordinates

############################### GEOMETR FOR CANTILEVER WIRES ###########################

coordSign = [1,1,-1,-1] # Left wires are in +z-direction, right wires are in -z-direction
x_offset = 0.2          # Distance from CW/MW to where the walled wire is connected to the cantilever. NB: MÅÅÅÅÅÅÅÅÅÅÅÅÅL 

cantWires = list()
cantWireNames=['Left CW cantilever wire','Left MW cantilever wire','Right CW cantilever wire','Right MW cantilever wire']
for i in range(0,len(cantCoordinates)):
	cantWires.append([[cantCoordinates[i][0][0][0],cantCoordinates[i][0][0][1],cantCoordinates[i][0][0][2]+4.6*coordSign[i]],\
                         [cantCoordinates[i][0][1][0]-x_offset,cantCoordinates[i][0][1][1],cantCoordinates[i][0][0][2]]])

############################### END OF GEOMETRY-SCRIPT #################################

############################### GEOMETRY INPUTS ########################################
# Creating one part - 'Whole Model'
mdb.models['Model-1'].Part(dimensionality=THREE_D, name='Whole Model', type=
    DEFORMABLE_BODY)

#Messenger wire geometry (and reference point)
mdb.models['Model-1'].parts['Whole Model'].ReferencePoint(point=(0.0, 0.0, 0.0))
mdb.models['Model-1'].parts['Whole Model'].WirePolyLine(mergeType=IMPRINT, meshable=
    ON, points=(mwCoordinates))	

# Contact wire geometry
mdb.models['Model-1'].parts['Whole Model'].WirePolyLine(mergeType=IMPRINT, meshable=
    ON, points=(cwCoordinates))

# Dropper geometry
for i in range(0,(numDr)):
	mdb.models['Model-1'].parts['Whole Model'].WirePolyLine(mergeType=IMPRINT, meshable=
		ON, points=(drSkewGeometry[i]))

# Top anchor
mdb.models['Model-1'].parts['Whole Model'].WirePolyLine(mergeType=IMPRINT, meshable=
    ON, points=(taCoordinates))

# Cantilevered tube
mdb.models['Model-1'].parts['Whole Model'].WirePolyLine(mergeType=IMPRINT, meshable=
    ON, points=(ctCoordinates))

# Registration arm dropper
mdb.models['Model-1'].parts['Whole Model'].WirePolyLine(mergeType=IMPRINT, meshable=
    ON, points=(radCoordinates))
	
# Registration arm
mdb.models['Model-1'].parts['Whole Model'].WirePolyLine(mergeType=IMPRINT, meshable=
    ON, points=(raCoordinates))
	
# Steady arm
mdb.models['Model-1'].parts['Whole Model'].WirePolyLine(mergeType=IMPRINT, meshable=
    ON, points=(saCoordinates))

# Cantilevers - NB!!! must use [] around the cantCoordinates to extract the correct tuple for Abaqus (NOT ANY MORE!)
for i in range(0,len(cantCoordinates)):
	mdb.models['Model-1'].parts['Whole Model'].WirePolyLine(mergeType=IMPRINT, meshable=
		ON, points=(cantCoordinates[i]))

# Cantilever wires
for i in range(0,len(cantWires)):
	mdb.models['Model-1'].parts['Whole Model'].WirePolyLine(mergeType=IMPRINT, meshable=
		ON, points=(cantWires[i]))


######################## SETS ################################################
# Messenger wire
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt((mwMidpoint[0], ), (mwMidpoint[1], ), (mwMidpoint[2], 
    ), (mwMidpoint[3], ), (mwMidpoint[4], ), (mwMidpoint[5], ), ), name='Messenger wire')

# Contact wire 
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt((cwMidpoint[0], ), (cwMidpoint[1], ), (cwMidpoint[2], 
    ), (cwMidpoint[3], ), (cwMidpoint[4], ), (cwMidpoint[5], ), ), name='Contact wire')

# Top anchor
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt((taMidpoint[0], ), (taMidpoint[1], ), (taMidpoint[2], 
    ),), name='Top anchor')

#### Cantilevers
#Left CW Cantilever
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt((cantCoordinates[0][0][0], ),
	(cantCoordinates[0][-1][-1], ), (cantWires[0][1], ),), name=cantNames[0])
#Left MW cantilever
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt((cantCoordinates[1][0][0], ),
	(cantCoordinates[1][-1][-1], ), (cantWires[1][1], ),), name=cantNames[1])
#Right CW cantilever
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt((cantCoordinates[2][0][0], ),
	(cantCoordinates[2][-1][-1], ), (cantWires[2][1], ),), name=cantNames[2])
#Right CW cantilever
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt((cantCoordinates[3][0][0], ),
	(cantCoordinates[3][-1][-1], ), (cantWires[3][1], ),), name=cantNames[3])
	
#### Cantilevered wires
for i in range(0,len(cantWires)):
	mdb.models['Model-1'].parts['Whole Model'].Set(edges=
		mdb.models['Model-1'].parts['Whole Model'].edges.findAt((cantWires[i][0],\
		),), name=cantWireNames[i])

# Cantilevered tube
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt((ctMidpoint[0], ), (ctMidpoint[1], ),),  name='Cantilevered tube')

# Registration arm dropper - MODYFIED
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt((radCoordinates[0], ),
	(radCoordinates[1], ), (radCoordinates[2], ),(radCoordinates[3], ),
	(radCoordinates[4], ), (radCoordinates[5], ), (radCoordinates[6], ),
	(radCoordinates[7], ), (radCoordinates[8], ), (radCoordinates[9], ),),  name='Registration arm dropper')

# Registration arm
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt((raMidpoint[0], ), (raMidpoint[1], ), (raMidpoint[2], 
    ), (raMidpoint[3], ),), name='Registration arm')

# Steady arm
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt((saMidpoint[0], ),),  name='Steady arm')

#Dropper sets
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt(
	(dr1Midpoint[0], ), (dr1Midpoint[1], ), (dr1Midpoint[2], ),
	(dr1Midpoint[3], ), (dr1Midpoint[4], ), (dr1Midpoint[5], ), (dr1Midpoint[6], ),
	(dr1Midpoint[7], ), (dr1Midpoint[8], ), (dr1Midpoint[9], ), (dr1Midpoint[10], ),
	(dr1Midpoint[11], ), (dr1Midpoint[12], ), (dr1Midpoint[13], ), (dr1Midpoint[14], ),
	(dr1Midpoint[15], ), (dr1Midpoint[16], ), (dr1Midpoint[17], ), (dr1Midpoint[18], ),
	(dr1Midpoint[19], ), (dr1Midpoint[20], ), (dr1Midpoint[21], ), (dr1Midpoint[22], ),
	(dr1Midpoint[23], ), (dr1Midpoint[24], ), (dr1Midpoint[25], ), (dr1Midpoint[26], ),
	(dr1Midpoint[27], ), (dr1Midpoint[28], ), (dr1Midpoint[29], ), (dr1Midpoint[30], ),
	(dr1Midpoint[31], ), (dr1Midpoint[32], ), (dr1Midpoint[33], ), (dr1Midpoint[34], ),
	(dr1Midpoint[35], ), (dr1Midpoint[36], ), (dr1Midpoint[37], ), (dr1Midpoint[38], ), 
	(dr1Midpoint[39], ), (dr1Midpoint[40], ), (dr1Midpoint[41], ), (dr1Midpoint[42], ), 
	(dr1Midpoint[43], ), (dr1Midpoint[44], ), (dr1Midpoint[45], ), (dr1Midpoint[46], ), 
	(dr1Midpoint[47], ), (dr1Midpoint[48], ), (dr1Midpoint[49], ), ), name=
    'Dropper 1')
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt(
	(dr2Midpoint[0], ),(dr2Midpoint[1], ), (dr2Midpoint[2], ),
	(dr2Midpoint[3], ), (dr2Midpoint[4], ), (dr2Midpoint[5], ), (dr2Midpoint[6], ),
	(dr2Midpoint[7], ), (dr2Midpoint[8], ), (dr2Midpoint[9], ), (dr2Midpoint[10], ),
	(dr2Midpoint[11], ), (dr2Midpoint[12], ), (dr2Midpoint[13], ), (dr2Midpoint[14], ),
	(dr2Midpoint[15], ), (dr2Midpoint[16], ), (dr2Midpoint[17], ), (dr2Midpoint[18], ),
	(dr2Midpoint[19], ), (dr2Midpoint[20], ), (dr2Midpoint[21], ), (dr2Midpoint[22], ),
	(dr2Midpoint[23], ), (dr2Midpoint[24], ), (dr2Midpoint[25], ), (dr2Midpoint[26], ),
	(dr2Midpoint[27], ), (dr2Midpoint[28], ), (dr2Midpoint[29], ), (dr2Midpoint[30], ),
	(dr2Midpoint[31], ), (dr2Midpoint[32], ), (dr2Midpoint[33], ), (dr2Midpoint[34], ),
	(dr2Midpoint[35], ), (dr2Midpoint[36], ), (dr2Midpoint[37], ), (dr2Midpoint[38], ), 
	(dr2Midpoint[39], ), (dr2Midpoint[40], ), (dr2Midpoint[41], ), (dr2Midpoint[42], ), 
	(dr2Midpoint[43], ), (dr2Midpoint[44], ), (dr2Midpoint[45], ), (dr2Midpoint[46], ), 
	(dr2Midpoint[47], ), (dr2Midpoint[48], ), (dr2Midpoint[49], ), ), name=
    'Dropper 2')
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt(
	(dr3Midpoint[0], ), (dr3Midpoint[1], ), (dr3Midpoint[2], ),
	(dr3Midpoint[3], ), (dr3Midpoint[4], ), (dr3Midpoint[5], ), (dr3Midpoint[6], ),
	(dr3Midpoint[7], ), (dr3Midpoint[8], ), (dr3Midpoint[9], ), (dr3Midpoint[10], ),
	(dr3Midpoint[11], ), (dr3Midpoint[12], ), (dr3Midpoint[13], ), (dr3Midpoint[14], ),
	(dr3Midpoint[15], ), (dr3Midpoint[16], ), (dr3Midpoint[17], ), (dr3Midpoint[18], ),
	(dr3Midpoint[19], ), (dr3Midpoint[20], ), (dr3Midpoint[21], ), (dr3Midpoint[22], ),
	(dr3Midpoint[23], ), (dr3Midpoint[24], ), (dr3Midpoint[25], ), (dr3Midpoint[26], ),
	(dr3Midpoint[27], ), (dr3Midpoint[28], ), (dr3Midpoint[29], ), (dr3Midpoint[30], ),
	(dr3Midpoint[31], ), (dr3Midpoint[32], ), (dr3Midpoint[33], ), (dr3Midpoint[34], ),
	(dr3Midpoint[35], ), (dr3Midpoint[36], ), (dr3Midpoint[37], ), (dr3Midpoint[38], ), 
	(dr3Midpoint[39], ), (dr3Midpoint[40], ), (dr3Midpoint[41], ), (dr3Midpoint[42], ), 
	(dr3Midpoint[43], ), (dr3Midpoint[44], ), (dr3Midpoint[45], ), (dr3Midpoint[46], ), 
	(dr3Midpoint[47], ), (dr3Midpoint[48], ), (dr3Midpoint[49], ), ), name=
    'Dropper 3')
mdb.models['Model-1'].parts['Whole Model'].Set(edges=
    mdb.models['Model-1'].parts['Whole Model'].edges.findAt(
	(dr4Midpoint[1], ), (dr4Midpoint[2], ),
	(dr4Midpoint[3], ), (dr4Midpoint[4], ), (dr4Midpoint[5], ), (dr4Midpoint[6], ),
	(dr4Midpoint[7], ), (dr4Midpoint[8], ), (dr4Midpoint[9], ), (dr4Midpoint[10], ),
	(dr4Midpoint[11], ), (dr4Midpoint[12], ), (dr4Midpoint[13], ), (dr4Midpoint[14], ),
	(dr4Midpoint[15], ), (dr4Midpoint[16], ), (dr4Midpoint[17], ), (dr4Midpoint[18], ),
	(dr4Midpoint[19], ), (dr4Midpoint[20], ), (dr4Midpoint[21], ), (dr4Midpoint[22], ),
	(dr4Midpoint[23], ), (dr4Midpoint[24], ), (dr4Midpoint[25], ), (dr4Midpoint[26], ),
	(dr4Midpoint[27], ), (dr4Midpoint[28], ), (dr4Midpoint[29], ), (dr4Midpoint[30], ),
	(dr4Midpoint[31], ), (dr4Midpoint[32], ), (dr4Midpoint[33], ), (dr4Midpoint[34], ),
	(dr4Midpoint[35], ), (dr4Midpoint[36], ), (dr4Midpoint[37], ), (dr4Midpoint[38], ), 
	(dr4Midpoint[39], ), (dr4Midpoint[40], ), (dr4Midpoint[41], ), (dr4Midpoint[42], ), 
	(dr4Midpoint[43], ), (dr4Midpoint[44], ), (dr4Midpoint[45], ), (dr4Midpoint[46], ), 
	(dr4Midpoint[47], ), (dr4Midpoint[48], ), (dr4Midpoint[49], ), ), name=
    'Dropper 4')
	
# Command to get the sets arranged as wanted:
session.journalOptions.setValues(replayGeometry=COORDINATE, recoverGeometry=COORDINATE)

############################## END GEOMETRY INPUT ###########################################

############################## DROPPER SUPPRESSING ##########################################
for i in dropperSuppress:
	mdb.models['Model-1'].parts['Whole Model'].features[drSuppressable[i-1]].suppress()
############################## END DROPPER SUPPRESS ########################################
	
################################ MATERIAL INPUT #########################################
# Material properties MW, CW and droppers, aluminium and steel for support (S25/S20 existing values 2.feb2015)
# Gathered from textfile materialprop.txt - NOT MODIFYABLE!!
matName = np.genfromtxt('materialprop.txt',comments='#', dtype='str', usecols=0, delimiter=',')
matEmodule = np.genfromtxt('materialprop.txt',comments='#', dtype='float', usecols=1, delimiter=',')
matDensity = np.genfromtxt('materialprop.txt',comments='#', dtype='float', usecols=2, delimiter=',')
matPoisson = np.genfromtxt('materialprop.txt',comments='#', dtype='float', usecols=3, delimiter=',')
matGmodule = [0]*len(matEmodule) 		# Defining shear modulus, to use in generalized profile

for j in range(len(matName)):
	matGmodule[j] = matEmodule[j]/(2*(1+matPoisson[j]))
	mdb.models['Model-1'].Material(name=matName[j])
	mdb.models['Model-1'].materials[matName[j]].Elastic(table=((
	matEmodule[j], matPoisson[j]), ))
	mdb.models['Model-1'].materials[matName[j]].Density(table=((matDensity[j], 
	), ))
	mdb.models['Model-1'].materials[matName[j]].Expansion(table=((17E-6, ), ))
####################### END MATERIAL INPUT ###############################################

####################### PROFILES INPUT ############################
# Creating section name and profile name the same: from section_MaterialAndName.txt
sectionName = np.genfromtxt('section_MaterialAndName.txt',comments='#', dtype='str', usecols=1, delimiter=',')
#CW
mdb.models['Model-1'].GeneralizedProfile(area=0.00012097, gammaO=0.0, gammaW=
    0.0, i11=1.3442e-10, i12=0.0, i22=1.1189e-10, j=1.8446e-09, name=sectionName[0])
#MW
mdb.models['Model-1'].CircularProfile(name=sectionName[1], r=0.00525)
#Droppers (1-4 are the same)
mdb.models['Model-1'].CircularProfile(name=(sectionName[2]), r=0.00225)

#Top anchor
mdb.models['Model-1'].PipeProfile(formulation=THICK_WALL, name=sectionName[3], r=
    0.0275, t=0.0065)
#Cantilevered tube
mdb.models['Model-1'].PipeProfile(formulation=THICK_WALL, name=
    sectionName[4], r=0.0275, t=0.0065)
# Registration arm
mdb.models['Model-1'].PipeProfile(formulation=THICK_WALL, name=
    sectionName[5], r=0.021, t=0.004)
# Reg arm dropper
mdb.models['Model-1'].CircularProfile(name=sectionName[6], r=0.003)
# Steady arm
mdb.models['Model-1'].RectangularProfile(a=0.025, b=0.013, name=sectionName[7])

# Cantilevers
mdb.models['Model-1'].GeneralizedProfile(area=0.0038364, gammaO=0.0, gammaW=
    0.0, i11=3.731401e-06, i12=0.0, i22=5.810597e-06, j=1.5e-05, name='Cantilever')



####################### END PROFILE INPUT #################################################

####################### SECTION DATA (all except contact wire) ############################
# Gathered from textfile section_MaterialAndName.txt - NB see for-loop: Contact wire is omitted
sectionMaterial = np.genfromtxt('section_MaterialAndName.txt',comments='#', dtype='str', usecols=0, delimiter=',')
sectionPoisson = np.genfromtxt('section_MaterialAndName.txt',comments='#', dtype='float', usecols=2, delimiter=',')
expansionCoeff = 1.7e-05 	#Same for CW/Dropper/MW
for i in range(1,len(sectionName)):
	if (i == 2):
		for j in range(1,numDr+1):
			mdb.models['Model-1'].BeamSection(consistentMassMatrix=False, integration=
				DURING_ANALYSIS, material=sectionMaterial[i], name=sectionName[i]+' ' + str(j), 
				poissonRatio=sectionPoisson[i], profile=sectionName[i], temperatureVar=LINEAR)
	else:
		mdb.models['Model-1'].BeamSection(consistentMassMatrix=False, integration=
				DURING_ANALYSIS, material=sectionMaterial[i], name=sectionName[i], 
				poissonRatio=sectionPoisson[i], profile=sectionName[i], temperatureVar=LINEAR)

####################### SECTION DATA FOR CONTACT WIRE: GENERALIZED DATA ###################
mdb.models['Model-1'].BeamSection(alphaDamping=0.0, beamShape=CONSTANT, 
    betaDamping=0.0, centroid=(0.0, 0.0), compositeDamping=0.0, 
    consistentMassMatrix=False, density=matDensity[1], dependencies=0, integration=
    BEFORE_ANALYSIS, name=sectionName[0], poissonRatio=matPoisson[1], profile=
    sectionName[0], shearCenter=(0.0, 0.0), table=((matEmodule[1], matGmodule[1],expansionCoeff,), ), 
	temperatureDependency=OFF, thermalExpansion=ON)
###################### SECTION FOR CANTILEVERS - generalized ###############################
mdb.models['Model-1'].BeamSection(alphaDamping=0.0, beamShape=CONSTANT, 
    betaDamping=0.0, centroid=(0.0, 0.0), compositeDamping=0.0, 
    consistentMassMatrix=False, density=matDensity[3], dependencies=0, integration=
    BEFORE_ANALYSIS, name='Cantilever', poissonRatio=matPoisson[3], profile=
    'Cantilever', shearCenter=(0.0, 0.0), table=((matEmodule[3], matGmodule[3]), ), 
    temperatureDependency=OFF, thermalExpansion=OFF)

###################### SECTION DATA FOR CANTILEVER WIRES ###################################
# Do not need, use the section for the messenger wire

###################### END SECTION DATA ####################################################

###################### SECTION ASSIGNMENT AND BEAM ORIENTATION #############################
#Second for loop is to get Dropper 1, Dropper 2 etc...
for i in range(1,len(sectionMaterial)):
	if (i == 2):
		for j in range(1,numDr+1):
			mdb.models['Model-1'].parts['Whole Model'].SectionAssignment(offset=0.0, 
				offsetField='', offsetType=MIDDLE_SURFACE, region=
				mdb.models['Model-1'].parts['Whole Model'].sets[sectionName[i]+' ' + str(j)], 
				sectionName=sectionName[i]+' ' + str(j), thicknessAssignment=FROM_SECTION)
	else:
		mdb.models['Model-1'].parts['Whole Model'].SectionAssignment(offset=0.0, 
				offsetField='', offsetType=MIDDLE_SURFACE, region=
				mdb.models['Model-1'].parts['Whole Model'].sets[sectionName[i]], 
				sectionName=sectionName[i], thicknessAssignment=FROM_SECTION)
for i in range(1,len(sectionName)):
	if (i==2):
		for j in range(1,numDr+1):
			mdb.models['Model-1'].parts['Whole Model'].assignBeamSectionOrientation(method=
				N1_COSINES, n1=(0.0, 0, -1.0), region=
				mdb.models['Model-1'].parts['Whole Model'].sets[sectionName[i]+' ' + str(j)])
	else:
		mdb.models['Model-1'].parts['Whole Model'].assignBeamSectionOrientation(method=
				N1_COSINES, n1=(0.0, 1.0, 0.0), region=
				mdb.models['Model-1'].parts['Whole Model'].sets[sectionName[i]])

# Contact wire - generalized, treated separately
mdb.models['Model-1'].parts['Whole Model'].SectionAssignment(offset=0.0, 
    offsetField='', offsetType=MIDDLE_SURFACE, region=
    mdb.models['Model-1'].parts['Whole Model'].sets['Contact wire'], 
    sectionName='Contact wire', thicknessAssignment=FROM_SECTION)
mdb.models['Model-1'].parts['Whole Model'].assignBeamSectionOrientation(method=
    N1_COSINES, n1=(0.0, 1.0, 0.0), region=
    mdb.models['Model-1'].parts['Whole Model'].sets['Contact wire'])

# Cantilevers - generalized cross section
for i in range(0,len(cantCoordinates)):
	mdb.models['Model-1'].parts['Whole Model'].SectionAssignment(offset=0.0, 
		offsetField='', offsetType=MIDDLE_SURFACE, region=
		mdb.models['Model-1'].parts['Whole Model'].sets[cantNames[i]], 
		sectionName='Cantilever', thicknessAssignment=FROM_SECTION)
	mdb.models['Model-1'].parts['Whole Model'].assignBeamSectionOrientation(method=
		N1_COSINES, n1=(0.0, 1.0, 0.0), region=
		mdb.models['Model-1'].parts['Whole Model'].sets[cantNames[i]])
# Cantilever wires:
for i in range(0,len(cantWires)):
	mdb.models['Model-1'].parts['Whole Model'].SectionAssignment(offset=0.0, 
		offsetField='', offsetType=MIDDLE_SURFACE, region=
		mdb.models['Model-1'].parts['Whole Model'].sets[cantWireNames[i]], 
		sectionName='Messenger wire', thicknessAssignment=FROM_SECTION)
	mdb.models['Model-1'].parts['Whole Model'].assignBeamSectionOrientation(method=
		N1_COSINES, n1=(0.0, 0.0, -1-0), region=
		mdb.models['Model-1'].parts['Whole Model'].sets[cantWireNames[i]])
###################### END SECTION ASSIGNMENT AND BEAM ORIENTATION ##########################

###################### CREATING INSTANCE AND MAKE INDEPENDENT ###############################
mdb.models['Model-1'].rootAssembly.DatumCsysByDefault(CARTESIAN)
mdb.models['Model-1'].rootAssembly.Instance(dependent=ON, name='Whole Model-1', 
    part=mdb.models['Model-1'].parts['Whole Model'])
mdb.models['Model-1'].rootAssembly.makeIndependent(instances=(
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'], ))

###################### ELEMENT TYPE - SAME FOR WHOLE MODEL - DEFAULT: SHEAR FLEXIBLE#########
#mdb.models['Model-1'].rootAssembly.setElementType(elemTypes=(ElemType(
#    elemCode=B33, elemLibrary=STANDARD), ), regions=(
#    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].edges.getSequenceFromMask(
#    ('[#7ffffff ]', ), ), ))

##################### SEED AND MESH INSTANCE/PART ###########################################
mdb.models['Model-1'].rootAssembly.seedPartInstance(deviationFactor=0.1, 
    minSizeFactor=0.1, regions=(
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'], ), size=0.2)
mdb.models['Model-1'].rootAssembly.generateMesh(regions=(
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'], ))
	
##################### SETS FOR BOUNDARY CONDITIONS #################################
# Set/BC name
bcName = ['Left Cantilever BC','Right end Cantilever BC','Midpoint support']
#Left end cantilevers
mdb.models['Model-1'].rootAssembly.Set(name=bcName[0], vertices=
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    (cantCoordinates[0][0][0], ), (cantCoordinates[1][0][0], ), ))
#Right end cantilevers
mdb.models['Model-1'].rootAssembly.Set(name=bcName[1], vertices=
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    (cantCoordinates[2][0][0], ), (cantCoordinates[3][0][0], ), ))
#Support
mdb.models['Model-1'].rootAssembly.Set(name=bcName[2], vertices=
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    (ctCoordinates[0][0], ), (taCoordinates[0][0], ), ))

### BC sets for cantilever wires
cantWiresBCName='Cantilever wires BC'
mdb.models['Model-1'].rootAssembly.Set(name=cantWiresBCName, vertices=
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    (cantWires[0][0], ), (cantWires[1][0], ), (cantWires[2][0], ), (cantWires[3][0],  ), ))
##################### CONNECTOR SECTIONS #####################################################
# Fixed
mdb.models['Model-1'].ConnectorSection(assembledType=WELD, name=
    'Fixed_connection')
mdb.models['Model-1'].ConnectorSection(name='Longitudinally fixed', 
    rotationalType=ALIGN, translationalType=SLOT)
mdb.models['Model-1'].sections['Longitudinally fixed'].setValues(
    behaviorOptions=(ConnectorStop(minMotion=-0.05, maxMotion=0.05, 
    components=(1, )), ))


##################### CONNECTOR WIRES AND SETS ################################################
#NB: findAt is taken as point to point from geometry file

# Contact wire-steady arm connection
mdb.models['Model-1'].rootAssembly.WirePolyLine(mergeType=IMPRINT, meshable=OFF, 
    points=((
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    cwCoordinates[3][0], ), 
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    saCoordinates[0][0], )), ))
mdb.models['Model-1'].rootAssembly.Set(edges=
    mdb.models['Model-1'].rootAssembly.edges.findAt((cwCoordinates[3][0], ))
    , name='Contact wire-steady arm')

# Top anchor - Messenger wire
mdb.models['Model-1'].rootAssembly.WirePolyLine(mergeType=IMPRINT, meshable=OFF, 
    points=((
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    taCoordinates[1][1], ), 
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    mwCoordinates[3][0], )), ))
mdb.models['Model-1'].rootAssembly.Set(edges=
    mdb.models['Model-1'].rootAssembly.edges.findAt((taCoordinates[1][1], )), 
    name='Top anchor - Messenger wire')

# Steady arm - Registration arm
mdb.models['Model-1'].rootAssembly.WirePolyLine(mergeType=IMPRINT, meshable=OFF
    , points=((
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    saCoordinates[0][0], ), 
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    raCoordinates[0][1], )), ))
mdb.models['Model-1'].rootAssembly.Set(edges=
    mdb.models['Model-1'].rootAssembly.edges.findAt((raCoordinates[0][1], 
    )), name='Steady arm - Registration arm')
########################## END CONNECTOR WIRES AND SETS ######################################

########################## CONNECTOR ASSIGNMENTS - NB changable!!! ###########################
# Fixed connections - which connections that are to be fixed (ONLY MESSENGER WIRE AND CONTACT WIRE).
fixed_connections_name = ['Contact wire-steady arm','Top anchor - Messenger wire']
for i in range(len(fixed_connections_name)):
	mdb.models['Model-1'].rootAssembly.SectionAssignment(region=
		mdb.models['Model-1'].rootAssembly.sets[fixed_connections_name[i]], sectionName=
		'Fixed_connection')
# Suppressing the fixed connections for the use of constraints instead:
mdb.models['Model-1'].rootAssembly.sectionAssignments[0].suppress()
mdb.models['Model-1'].rootAssembly.sectionAssignments[1].suppress()

# Steady arm stop
# Datum:
mdb.models['Model-1'].rootAssembly.DatumCsysByThreePoints(coordSysType=
    CARTESIAN, name='SteadyArm_RegistrationArm', origin=
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    saCoordinates[0][0], ), point1=(saCoordinates[0][0][0],saCoordinates[0][0][1]+1.0,saCoordinates[0][0][2]), point2=
    mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    raCoordinates[0][0], ))
#Assignment: Do manually


########################## BOUNDARY CONDITIONS ################################################
# Right and left end (longitudinal is free to move)
for i in range(len(bcName)-1):
	mdb.models['Model-1'].DisplacementBC(amplitude=UNSET, createStepName='Initial', 
		distributionType=UNIFORM, fieldName='', localCsys=None, name=bcName[i], 
		region=mdb.models['Model-1'].rootAssembly.sets[bcName[i]], u1=SET, u2=SET, 
		u3=SET, ur1=SET, ur2=SET, ur3=SET)
# Support: all fixed
mdb.models['Model-1'].DisplacementBC(amplitude=UNSET, createStepName='Initial', 
	distributionType=UNIFORM, fieldName='', localCsys=None, name=bcName[2], 
	region=mdb.models['Model-1'].rootAssembly.sets[bcName[2]], u1=SET, u2=SET, 
	u3=SET, ur1=SET, ur2=SET, ur3=SET)

# Cantilever wires - fixed
mdb.models['Model-1'].DisplacementBC(amplitude=UNSET, createStepName='Initial', 
	distributionType=UNIFORM, fieldName='', localCsys=None, name=cantWiresBCName, 
	region=mdb.models['Model-1'].rootAssembly.sets[cantWiresBCName], u1=SET, u2=SET, 
	u3=SET, ur1=SET, ur2=SET, ur3=SET)


########################## STEPS ###############################################################
# Static steps:
st_step = ['Initial','Pretension load','Gravity','Tension']
for i in range(1,len(st_step)):
	mdb.models['Model-1'].StaticStep(initialInc=0.01, maxInc=1, maxNumInc=100, 
		name=st_step[i], nlgeom=ON, previous=st_step[i-1])
# Frequency step
#mdb.models['Model-1'].FrequencyStep(name='Frequency', numEigen=8, previous=
#    'Tension')

########################## FIELD AND HISTORY OUTPUTS ############################################
#Field
mdb.models['Model-1'].fieldOutputRequests['F-Output-1'].setValues(variables=(
    'U', ))
mdb.models['Model-1'].FieldOutputRequest(createStepName='Tension', name=
    'F-Output-3', variables=('U', 'RF', 'NFORC', 'SF'))
#History
mdb.models['Model-1'].historyOutputRequests['H-Output-1'].setValues(variables=(
    'ALLAE', ))

########################## AMPLITUDES ###########################################################
# input from textfiles: arranged in [name_of_input,txt-file]
amplitude_inputs = ['amp_0_0,001.txt','gravity_0-1.txt','Tension_0,001-1.txt']
amplitude_names = ['amp_initial_input','gravity_0-1','tension_0,001-1']
amp_abaqus = [0]*len(amplitude_names)	# Must create a new vector for tuple !
amp_abaqus_real = [0]*len(amplitude_names)
for i in range(len(amplitude_inputs)):
	amplitude_inputs[i] = np.genfromtxt(amplitude_inputs[i], comments='#',dtype=float)
	amplitude_inputs[i] = amplitude_inputs[i].tolist()
	amp_abaqus[i] = [[0]*2]*len(amplitude_inputs[i])            # each "coordinate" to tuple
	amp_abaqus_real[i] = [[0]*2]*len(amplitude_inputs[i])
	for j in range(len(amplitude_inputs[i])):
		amp_abaqus[i][j] = tuple(amplitude_inputs[i][j])		# Tuple each "coordinate"
		amp_abaqus_real[i] = tuple(amp_abaqus[i])          		# Tuple each component (initial,gravity,tension)
	mdb.models['Model-1'].TabularAmplitude(data=(amp_abaqus_real[i]), 
	name=amplitude_names[i], smooth=SOLVER_DEFAULT, timeSpan=STEP)
# Constant gravity-load amplitude
mdb.models['Model-1'].TabularAmplitude(data=((0.0, 0.001), (1.0, 0.001)), name=
    'gravity-step load amp', smooth=SOLVER_DEFAULT, timeSpan=STEP)
	
######################## LOADS - Can edit this to get different magnitudes in MW/CW ##############
loadName = ['Tension left end','Tension right end','Gravity']
loadSets = ['Left CW end','Left MW end','Right CW end','Right MW end']
loadPosition = [cwCoordinates[0][0],mwCoordinates[0][0],cwCoordinates[-1][-1],mwCoordinates[-1][-1]]
####### Load sets ######
for i in range(0,len(cantCoordinates)):
	mdb.models['Model-1'].rootAssembly.Set(name=loadSets[i], vertices=
		mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
		(loadPosition[i], ), ))

# Tensioning load left and right end - See st_step,bcName
magnitudeLoad = 10000 # Magnitude of the applied load in both wires
loadWires = [magnitudeLoad,magnitudeLoad,-magnitudeLoad,-magnitudeLoad]
for i in range(0,len(loadPosition)):
	mdb.models['Model-1'].ConcentratedForce(amplitude=amplitude_names[0], cf3=loadWires[i], 
		createStepName=st_step[1], distributionType=UNIFORM, field='', 
		localCsys=None, name=loadSets[i], region=
		mdb.models['Model-1'].rootAssembly.sets[loadSets[i]])
	mdb.models['Model-1'].loads[loadSets[i]].setValuesInStep(amplitude=
    'gravity-step load amp', stepName='Gravity')
	#mdb.models['Model-1'].loads[loadSets[i]].setValuesInStep(amplitude=
    #amplitude_names[2], stepName='Tension')

# Gravity load - Comment: could be appplied incrementally aswell (IS DONE)
mdb.models['Model-1'].Gravity(comp2=-0.0981, createStepName='Gravity', 
    distributionType=UNIFORM, field='', name='Gravity load',amplitude=amplitude_names[1])
mdb.models['Model-1'].loads['Gravity load'].setValuesInStep(amplitude=FREED, 
    stepName='Tension')

########################## CONSTRAINTS AT SUPPORT (HINGES) ###################################
constr_names = ['RegArm_CantTube hinge','CantTube_TopAnchor hinge','CantTube_RegArmDropper hinge',\
				'RegArmDropper_RegArm hinge','SteadyArm_RegArm hinge']

# Registration arm - Cantilevered tube hinge 
mdb.models['Model-1'].Coupling(controlPoint=Region(
	vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
	(raCoordinates[0][0], ), )), couplingType=KINEMATIC, influenceRadius=
	WHOLE_SURFACE, localCsys=None, name=constr_names[0], surface=
	Region(
	vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
	(ctCoordinates[0][1], ), )), u1=ON, u2=ON, u3=ON, ur1=ON, ur2=ON, ur3=OFF)
# Cantilevered tube - Top anchor
mdb.models['Model-1'].Coupling(controlPoint=Region(
	vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
	(ctCoordinates[1][1], ), )), couplingType=KINEMATIC, influenceRadius=
	WHOLE_SURFACE, localCsys=None, name=constr_names[1], surface=
	Region(
	vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
	(taCoordinates[0][1], ), )), u1=ON, u2=ON, u3=ON, ur1=ON, ur2=ON, ur3=OFF)
# Cantilevered tube - registration arm dropper
mdb.models['Model-1'].Coupling(controlPoint=Region(
	vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
	(radCoordinates[0], ), )), couplingType=KINEMATIC, influenceRadius=
	WHOLE_SURFACE, localCsys=None, name=constr_names[2], surface=
	Region(
	vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
	(ctCoordinates[1][1], ), )), u1=ON, u2=ON, u3=ON, ur1=ON, ur2=OFF, ur3=OFF)
# Registration arm dropper - Registration arm
mdb.models['Model-1'].Coupling(controlPoint=Region(
	vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
	(radCoordinates[-1], ), )), couplingType=KINEMATIC, influenceRadius=
	WHOLE_SURFACE, localCsys=None, name=constr_names[3], surface=
	Region(
	vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
	(raCoordinates[1][1], ), )), u1=ON, u2=ON, u3=ON, ur1=OFF, ur2=OFF, ur3=OFF)
#Steady arm - Registration arm
mdb.models['Model-1'].Coupling(controlPoint=Region(
	vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
	(saCoordinates[0][1], ), )), couplingType=KINEMATIC, influenceRadius=
	WHOLE_SURFACE, localCsys=None, name=constr_names[4], surface=
	Region(
	vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
	(raCoordinates[2][1], ), )), u1=ON, u2=ON, u3=ON, ur1=OFF, ur2=OFF, ur3=OFF)
	
########################## CONSTRAINTS FOR CW AND MW ##########################################
# Steady arm - Contact wire
mdb.models['Model-1'].Coupling(controlPoint=Region(
    vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    (saCoordinates[0][0], ), )), couplingType=KINEMATIC, influenceRadius=
    WHOLE_SURFACE, localCsys=None, name='SteadyArm_CW', surface=Region(
    vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    (cwCoordinates[3][0], ), )), u1=ON, u2=ON, u3=ON, ur1=OFF, ur2=ON, ur3=ON)

# Top anchor - Messenger wire
mdb.models['Model-1'].Coupling(controlPoint=Region(
    vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    (taCoordinates[2][0], ), )), couplingType=KINEMATIC, influenceRadius=
    WHOLE_SURFACE, localCsys=None, name='TopAnchor_MW', surface=Region(
    vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    (mwCoordinates[3][0], ), )), u1=ON, u2=ON, u3=OFF, ur1=OFF, ur2=ON, ur3=ON)


########################## CONSTTRAINTS CANTILEVERS _ WIRES ###################################
cantConstrName = ['Left CW tie','Left MW tie','Right CW tie','Right MW tie']
for i in range(0,len(cantConstrName)):
	mdb.models['Model-1'].Coupling(controlPoint=Region(
    vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    (cantCoordinates[i][0][1], ), )), couplingType=KINEMATIC, influenceRadius=
    WHOLE_SURFACE, localCsys=None, name=cantConstrName[i], surface=Region(
    vertices=mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].vertices.findAt(
    (loadPosition[i], ), )), u1=ON, u2=ON, u3=ON, ur1=ON, ur2=ON, 
    ur3=ON)
	
#################### PREDEFINED FIELDS (TEMPERATURE)###########################################
tempMagnitude = [-81,-188] #[CW,MW]=[-81,-188] Gives approx 10kN in both wires (with slack geometry)
tempSets = ['Contact wire','Messenger wire'] 				#from earlier sets
fieldName = ['CW temperature field','MW temperature field']	#new field name
for i in range(0,2):
	mdb.models['Model-1'].Temperature(createStepName='Tension', 
		crossSectionDistribution=CONSTANT_THROUGH_THICKNESS, distributionType=
		UNIFORM, magnitudes=(tempMagnitude[i], ), name=fieldName[i], region=
		mdb.models['Model-1'].rootAssembly.instances['Whole Model-1'].sets[tempSets[i]])
	#mdb.models['Model-1'].predefinedFields[fieldName[i]].setValues(
	#	amplitude='gravity_0-1')


		
######################## GENERAL CONTACT #######################################################
# Interaction properties
mdb.models['Model-1'].ContactProperty('GeneralContactProp')
mdb.models['Model-1'].interactionProperties['GeneralContactProp'].TangentialBehavior(
    formulation=FRICTIONLESS)
mdb.models['Model-1'].interactionProperties['GeneralContactProp'].NormalBehavior(
    allowSeparation=ON, constraintEnforcementMethod=DEFAULT, 
    pressureOverclosure=HARD)
mdb.models['Model-1'].interactionProperties['GeneralContactProp'].GeometricProperties(
    contactArea=1.0, padThickness=None)

########################################## JOB #################################################
mdb.Job(atTime=None, contactPrint=OFF, description='', echoPrint=OFF, 
    explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
    memory=90, memoryUnits=PERCENTAGE, model='Model-1', modelPrint=OFF, 
    multiprocessingMode=DEFAULT, name='FirstJob', nodalOutputPrecision=SINGLE, 
    numCpus=1, numGPUs=0, queue=None, scratch='', type=ANALYSIS, 
    userSubroutine='', waitHours=0, waitMinutes=0)


		######################## KEYWORDS - Run keywords.py ###########
		###############################################################
# Function for counting lines in inp-file (keywords):
modelName = 'Model-1'
def GetKeywordPosition(modelName, blockPrefix, occurrence=1):
    if blockPrefix == '':
        return len(mdb.models[modelName].keywordBlock.sieBlocks)-1
    pos = 0
    foundCount = 0
    for block in mdb.models[modelName].keywordBlock.sieBlocks:
        if str.lower(block[0:len(blockPrefix)])==\
           str.lower(blockPrefix):
            foundCount = foundCount + 1
            if foundCount >= occurrence:
                return pos
        pos=pos+1
    return -1

#execute files "keywords---.py" to get expansion and beam to beam contact 



