# Script to run several jobs, with edited temperatures
from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *

import numpy as np

# Values for CW=10kN, MW=5-15kN with 0.5kN interval - OBTAINED FROM tensionTemp10kN.m (In Abaqus CAE results folder)
tensionInterval = ['5','5,5','6','6,5','7','7,5','8','8,5','9','9,5','10','10,5',
				   '11','11,5','12','12,5','13','13,5','14','14,5','15','15,5',
				   '16','16,5','17','17,5','18','18,5','19','19,5','20']

########## EDITED FOR EACH MODEL: ###############
# Check the temperature-files and copy the name of them into 'modelName'
modelName = 'Temp-Constant-tension-CW=10kN'	# EDIT THIS
mdb.Model(name=modelName, objectToCopy=mdb.models['Model-1'])

# Choose which wire that is fast and which has an interval-tension
cwTension='10'									#EDITABLE
mwTension=tensionInterval						#EDITABLE

########## Importing temperature #################
tempFile = modelName+'.txt'
tempchange = np.genfromtxt(tempFile, delimiter='')
nTemp = len(tempchange)

cw_temperature = [0]*nTemp
mw_temperature = [0]*nTemp

for i in range(0,nTemp):
	cw_temperature[i]=tempchange[i][0]
	mw_temperature[i]=tempchange[i][1]
###################################################

# Creating job names
jobNames = []
for i in range(0,len(tensionInterval)):
	if len(cwTension) != 31:
		jobNames.append('CW'+cwTension+'-MW'+mwTension[i])
	elif len(cwTension) ==31 and len(mwTension) == 31:
		jobNames.append('CW'+cwTension[i]+'-MW'+mwTension[i])
	else:
		jobNames.append('CW'+cwTension[i]+'-MW'+mwTension)
		


nJobs = len(jobNames)

for i in range(0,nJobs):
	mdb.Job(atTime=None, contactPrint=OFF, description='', echoPrint=OFF, 
		explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
		memory=90, memoryUnits=PERCENTAGE, model=modelName, modelPrint=OFF, 
		multiprocessingMode=DEFAULT, name=jobNames[i], nodalOutputPrecision=SINGLE, 
		numCpus=1, numGPUs=0, queue=None, resultsFormat=ODB, scratch='', type=
		ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)
	mdb.models[modelName].predefinedFields['CW temperature field'].setValues(
		magnitudes=(cw_temperature[i], ))
	mdb.models[modelName].predefinedFields['MW temperature field'].setValues(
		magnitudes=(mw_temperature[i], ))
	
	mdb.jobs[jobNames[i]].submit(consistencyChecking=OFF)
	mdb.jobs[jobNames[i]].waitForCompletion()
	execfile('EigenFrequencyOutputALL.py')